Attribute VB_Name = "General_Routines"
Option Explicit

Private Const OF_READ = &H0&
Private Declare Function lOpen Lib "kernel32" Alias "_lopen" (ByVal lpPathName As String, ByVal iReadWrite As Long) As Long
Private Declare Function lClose Lib "kernel32" Alias "_lclose" (ByVal hFile As Long) As Long
Private Declare Function GetFileSize Lib "kernel32" (ByVal hFile As Long, lpFileSizeHigh As Long) As Long
Public Function FileSize(sFilePath As String) As Long

  Dim lPointer As Long
  Dim lFileSize As Long
  Dim lpFSHigh As Long
  
  lPointer = lOpen(sFilePath, OF_READ)
    lFileSize = GetFileSize(lPointer, lpFSHigh)   'size of the file
  lClose lPointer
  
'  Debug.Print lFileSize & " bytes"
  
  FileSize = lFileSize
  
End Function
Public Function HexToLong(ByVal sHex As String) As Long
'Converts a string that contains a HEX number, up to 7 digits, to a Long
'Even handles negative hex

  Dim l As Long
  Dim lSign As Long

  If Left(sHex, 1) = "-" Then
    sHex = Mid(sHex, 2)
    lSign = -1
  Else
    lSign = 1
  End If

  ' Base Case, Empty string or non-hex characters
  If Not IsHex(sHex) Then
    l = -1
    
  ' Anything bigger than 7 characters can cause an overflow
  ElseIf Len(sHex) > 7 Then
    l = -2
    
  Else
    l = "&H" & sHex
  
  End If
  
  HexToLong = l * lSign

End Function
Public Function IsHex(ByVal sHex As String) As Boolean
'Returns True IFF the string passed to it is a HEX number of 7 digits or less
'Empty string now returns False

  Dim I As Integer
  Dim iIndex As Integer

  'Empty string returns False
  If sHex = "" Then
    IsHex = False
    Exit Function
  End If
  
  For iIndex = 1 To Len(sHex)
  
    I = Asc(UCase(Mid(sHex, iIndex, 1)))
    
    ' 0..... 9   A... F
    '48     57  65   70
    If I < 48 Or (I > 57 And I < 65) Or I > 70 Then
      IsHex = False
      Exit Function
    End If
    
  Next iIndex
    
  IsHex = True
  
End Function
Public Function IsSignedHex(ByVal sHex As String) As Boolean
'Returns True IFF the string passed to it is a HEX number of 7 digits or less
'or the same thing with a - in front of it
'Empty string returns False

  'Empty string returns False
  If sHex <> "" Then
    If Left(sHex, 1) = "-" Then sHex = Mid(sHex, 2)
  End If
  
  IsSignedHex = IsHex(sHex)
  
End Function
Public Function IsFloat(ByVal sFloat As String) As Boolean
'Returns True IFF the string passed to it is a Float number
'Empty string now returns False

  Dim I As Integer
  Dim iIndex As Integer
  Dim bFoundPoint As Boolean

  If sFloat = "." Or sFloat = "" Then
    IsFloat = False
    Exit Function
  End If

  For iIndex = 1 To Len(sFloat)
  
    I = Asc(UCase(Mid(sFloat, iIndex, 1)))
    
    If (I < 48 Or I > 57) And (I <> 46 Or bFoundPoint) Then
      IsFloat = False
      Exit Function
    End If
    
    If I = 46 Then bFoundPoint = True
    
  Next
  
  IsFloat = True
    
End Function
Public Function TextFileType(ByVal iFileNum As Integer, bErr As Boolean) As Integer
' Added in version 12.8.12
' Determines which type of text file we are dealing with based on the termination of the first line.
' If the file is only one line, it is considered to be a PC file
' 1 PC/DOS = CR LF (13 10)
' 2 Max    = CR
' 3 Unix   = LF

  Dim s As String
  
  TextFileType = 1 ' PC
  
  ' If there is no bytes in the file at all, this should be an error
  If Loc(iFileNum) >= LOF(iFileNum) Then bErr = True: Exit Function
  
  ' Look for the first line terminator
  ' If the file ends on the first line, just say it's a PC file
  bErr = False
  s = Input(1, #iFileNum)
  Do While s <> vbLf And s <> vbCr
    If Loc(iFileNum) >= LOF(iFileNum) Then Exit Function
    s = Input(1, #iFileNum)
  Loop
  
  If s = vbLf Then
    TextFileType = 3 ' UNIX - LF first
  Else
    If Input(1, #iFileNum) <> vbLf Then TextFileType = 2 ' Mac - CR followed by a LF
    ' Otherwise it's a PC file
  End If
    
End Function
Public Function ReadLine(ByVal iFileNum As Integer, ByVal iTextFileType As Integer, bErr As Boolean) As String
' Added in version 12.8.12
' Designed to do the same as: Line Input #iFileNum, sRow
' but to handle non-DOS formatted text files
' 1 PC/DOS = CR LF
' 2 Mac    = CR
' 3 Unix   = LF

  Dim s As String
  Dim sRow As String: sRow = ""
  Dim sTerm As String
  
  If iTextFileType = 3 Then
    sTerm = vbLf
  Else
    sTerm = vbCr
  End If
  
  ' If there is no bytes left in the file at all, this should be an error
  If Loc(iFileNum) >= LOF(iFileNum) Then bErr = True: GoTo Err_ReadLine
  
  ' Otherwise, concat every byte that isn't the first terminator
  ' If the file ends during this line, just pass back what there is of the line
  bErr = False
  s = Input(1, #iFileNum)
  Do While s <> sTerm
    sRow = sRow & s
    If Loc(iFileNum) >= LOF(iFileNum) Then GoTo Err_ReadLine
    s = Input(1, #iFileNum)
  Loop
  
  ' If it's a PC file, then remove the remaining LF
  If iTextFileType = 1 Then
    If Loc(iFileNum) >= LOF(iFileNum) Then bErr = True: GoTo Err_ReadLine
    s = Input(1, #iFileNum)
    If s <> vbLf Then bErr = True
  End If
  
Err_ReadLine:
  
  'Debug.Print bErr & "'" & sRow & "'"
  ReadLine = sRow

End Function
Public Function ExtractString(ByVal sRow As String, ByRef lIndex As Long) As String
' Extract sub-string that ends with termination string gsTerm

  Dim sChar As String
  Dim sString As String
  Dim sTerm As String
  
  Dim lIndexStart As Long
  Dim lIndexTerm As Long

  Dim gsTerm As String: gsTerm = Chr(9) 'Tab

  If lIndex < Len(sRow) Then ' Note: This should always be true...
    lIndexStart = lIndex + 1
    sString = ""
    
    ' --- Clear the termination string ---
    sTerm = ""
    lIndexTerm = 1
    
    Do
      ' --- Get 1 character ---
      lIndex = lIndex + 1
      sChar = Mid(sRow, lIndex, 1)
      
      If sChar = Mid(gsTerm, lIndexTerm, 1) Then
        ' --- Character is a termination character ---
        lIndexTerm = lIndexTerm + 1
        sTerm = sTerm & sChar
      Else
        ' --- Character is a not termination character ---
        ' --- thus previous characters were also not termination characters ---
        sString = sString & sTerm & sChar
        sTerm = ""
        lIndexTerm = 1
      End If

    Loop Until (sTerm = gsTerm) Or lIndex >= Len(sRow)
    
  End If

  ExtractString = sString

End Function
Public Function StripFolder(ByVal sFile As String) As String
' If you pass a file path it will return the folder path
' of that file path or an empty string if there is no \
' in the path
 
  Do Until Right(sFile, 1) = "\" Or Len(sFile) = 0
    sFile = Left(sFile, Len(sFile) - 1)
  Loop
  
  StripFolder = sFile
  
End Function
Public Function StripFile(ByVal sFile As String) As String
' If you pass a file path it will return the file name
' of that file path
 
  StripFile = Mid(sFile, Len(StripFolder(sFile)) + 1)
  
End Function
Public Function StripExtender(ByVal sFile As String) As String
' If you pass a file it will return the file without
' the extnder or the original file if there was no .
' in the file
 
  Dim sFileCopy As String: sFileCopy = sFile
 
  Do Until Right(sFileCopy, 1) = "." Or Len(sFileCopy) = 0
    sFileCopy = Left(sFileCopy, Len(sFileCopy) - 1)
  Loop
  
  If Len(sFileCopy) = 0 Then
    StripExtender = sFile
  Else
    StripExtender = Left(sFileCopy, Len(sFileCopy) - 1)
  End If
  
End Function
Public Sub Error_Handler(ByVal sErrorDescription As String, Optional ByVal sFileDir As String)

  ' Generic Visual BasicError Handler

  Dim sTemp As String
  Dim iDummy As Integer
  
  sTemp = ""
  If Len(sFileDir) > 0 Then sTemp = vbCrLf & vbCrLf & "Filename accessed -> " & sFileDir
  
  iDummy = MsgBox( _
    "Error #" & Err.Number & " -> " & Error(Err.Number) & vbCrLf & vbCrLf _
    & sErrorDescription _
    & sTemp, _
    vbCritical, "Error")
    
End Sub
Public Sub Minor_Error_Handler(ByVal sError As String, ByVal sErrorDescription As String, Optional sFileDir As String)

  ' Generic Visual BasicError Handler

  Dim sTemp As String
  Dim iDummy As Integer
  
  sTemp = ""
  If Len(sFileDir) > 0 Then sTemp = vbCrLf & vbCrLf & "Filename accessed -> " & sFileDir
  
  iDummy = MsgBox( _
    sErrorDescription _
    & sTemp, _
    vbExclamation, sError)
      
End Sub

